import sox
import numpy as np
import argparse
import glob
import os
from progress.bar import ChargingBar

def checkName(name):
    index = 1
    while os.path.isfile(name) == True:
        base = os.path.splitext(name)[0]
        name = base + "_" + str(index) + ".wav"
        index = index + 1
    else:
        return name
    
# PARSER

parser = argparse.ArgumentParser(description= "Batch process for databending every file to audio files", prog="Databending for audio")
parser.add_argument('--input', "-i", default= "/Applications/Ableton Live 11 Suite.app", help="Path to the folder containing the original files.")
parser.add_argument("--output", "-o", default="/Users/agustinissidoro/Desktop/MOSH AUDIO/Ableton Live", help= "Path where you want the bent files to go.")
parser.add_argument("--rate", "-r", default=48000, help="Sample rate")
parser.add_argument("--channels", "-c", default=1, help="Number of channels")
parser.add_argument("--encoding", "-e", default="signed-integer", help="Don't use this unless you are very confident. Refer to pysox docs.")
parser.add_argument("--bitdepth", "-b", default=16, help= "Bitdepth. Although in theory every integer value is possible, its avalability depends on the encoding param.")
parser.add_argument("--recursive", "-v", default=True, help="Looks for files in every subdirectory of the input path.")
parser.add_argument("--data", "-d", default="raw")
parser.add_argument("--ignore", "-a", default="", help="This particular file extension will not be bent. Include the dot like in '.wav'")
parser.add_argument("--delete", "-w", default= True, help= "Delete all files (not subfolders!) in output folder before processing. The code doesn't overwrite any file as it is desgined to rename in case of matching names.")

# INITIALIZE EVERYTHING----------------------------

args = parser.parse_args()

if args.delete == True:
    if os.path.isdir(args.output):
        for f in os.listdir(args.output):
            try:
                os.remove(f)
            except:
                print(f"Could not delete: {f}\n")
    else:
        pass
            
try:
    os.mkdir(args.output)
    print("\nOutput folder created.")
except:
    print("\nOutput folder already exists.")

n_files = sum([len(files) for r, d, files in os.walk(args.input)])
print(f"Total items in input folder: {n_files}")

bar = ChargingBar("Bending:", max=n_files, suffix = " ")

error_log = []

bend = sox.Transformer()
bend.set_input_format(rate= args.rate, bits = args.bitdepth, channels = args.channels, encoding = args.encoding, ignore_length=True)
bend.set_output_format(rate= args.rate, bits = args.bitdepth, channels = args.channels, encoding = args.encoding)

index = 0

# BENDING LOOP--------------------------------------

for f in glob.glob((args.input + "/**/**.*"), recursive=args.recursive):
    if os.path.isdir(f):
        pass
        bar.next()
    if os.path.splitext(f)[1] == args.ignore:
        pass
        bar.next()
    else:
        try:
            f_out = os.path.join(args.output, os.path.basename(f))
            f_name = os.path.splitext(f_out)[0] + ".wav"
            f_name = checkName(f_name)
            raw = np.fromfile(f, dtype=np.int16)
            status = bend.build_file(input_filepath=None, sample_rate_in=44100, input_array=raw, output_filepath=f_name)
            bar.next()
            index = index + 1
        except:
            error_log.append(f)
            pass #print(f"Could not convert{f}")

for i in range(n_files): #solo para completar la barra
    bar.next()
bar.finish()

# FINAL INFORMATION----------------------------------------------

print(f'{index} files bent.\n')

if len(error_log) != 0:
    print("The following files could not be converted: \n")
    for l in error_log:
        print(l)
else:
    pass
print("")
        
        